// Copyright 2020 Thomas Hintz
//
// This file is part of the Alpha Centauri Farming project.
//
// The Alpha Centauri Farming project is free software: you can
// redistribute it and/or modify it under the terms of the GNU General
// Public License as published by the Free Software Foundation, either
// version 3 of the License, or (at your option) any later version.
//
// The Alpha Centauri Farming project is distributed in the hope that
// it will be useful, but WITHOUT ANY WARRANTY; without even the
// implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
// PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with the Alpha Centauri Farming project.  If not, see
// <https://www.gnu.org/licenses/>.

import { GAME_STATES, ALERTS } from '../../constants.js'

import { batch } from 'react-redux'
import * as websocket from '../../websocket.js'

import { updateGame, updatePlayer, gameState, setSelectedCard, setCards,
         movePlayer, setOldMessages, markActionChangeHandled,
         mpMouse, rolled, setNextAction, nextUIAction, nextUIActionSilent, alert
       } from './actions.js'

export { initialize, buy, roll, endTurn, loan, trade, submitTradeAccept,
         submitTradeDeny, submitTradeCancel, audit, handleMessage,
         nextAction, buyUncleBert, actionsFinished }

let store;

let spacesWithPlayers = [];
let loop = 0;
function handleMessage(evt) {
    const data = JSON.parse(evt.data),
          type = data.event;

    if (data.event === 'error') {
        console.log('error:' + data.exn);
        return;
    }
    batch(() => {
        if (data.player.state === GAME_STATES.preTurn &&
            data.game.otherPlayers.length > 0 &&
            store.getState().farm.player.state !== GAME_STATES.preTurn) {
            store.dispatch(alert(ALERTS.beginTurn));
        } else if (data.game.otherPlayers.length > 0 &&
                   data.game.currentPlayer !== store.getState().farm.game.currentPlayer) {
            store.dispatch(alert(ALERTS.otherPlayersTurn));
        }
        store.dispatch(updatePlayer(data.player));
        if (data.event === 'init') {
            store.dispatch(movePlayer(data.player.space, 0, data.player.color));
        }
        // new player(s) added to game, put them on the board
        if (data.game.otherPlayers.length !== store.getState().farm.game.otherPlayers.length) {
            const otherPlayers = store.getState().farm.game.otherPlayers;
            const newPlayers = data.game.otherPlayers.filter(
                x => !otherPlayers.find(y => y.player.name === x.player.name));
            for (const p of newPlayers) {
                store.dispatch(movePlayer(p.player.space, 0, p.player.color));
            }
        }
        const oldMessages = store.getState().farm.game.messages.slice(0, 20);
        store.dispatch(updateGame(data.game));
        store.dispatch(setOldMessages(oldMessages));
        if (data.player.cards.length > 0) {
            store.dispatch(setSelectedCard(data.player.cards[0]));
        } else {
            store.dispatch(setSelectedCard());
        }
        store.dispatch(setCards(data.player.cards));
        if (data.event === 'action') {
            if (data.player.name !== data.game.currentPlayer &&
                data.action !== 'roll') {
                store.dispatch(nextUIAction());
            }
            store.dispatch(setNextAction(data.action, data.value));
            if (data.action === 'roll') {
                store.dispatch(nextUIAction());
            }
        }
        if (data.player.state === GAME_STATES.midTurn &&
            data.player.cash < 0 &&
            !store.getState().farm.ui.nextAction) {
            store.dispatch(alert(ALERTS.raiseMoney));
        }
    });
};

let sendCommand;

function buy(id, cash) {
    sendCommand({ type: 'buy', id: id, cash: cash });
}

function roll() {
    store.dispatch(setNextAction('pre-rolling', false));
    store.dispatch(nextUIActionSilent());
    sendCommand({ type: 'roll' });
}

function endTurn() {
    store.dispatch(gameState(GAME_STATES.turnEnded));
    sendCommand({ type: 'turn-ended' });
}

function loan(amount) {
    sendCommand({ type: 'loan', amount: amount });
}

function trade(parameters) {
    sendCommand({ type: 'trade',
                  parameters: parameters });
}

function submitTradeAccept() {
    sendCommand({ type: 'trade-accept' });
}

function submitTradeDeny() {
    sendCommand({ type: 'trade-deny' });
}

function submitTradeCancel() {
    sendCommand({ type: 'trade-cancel' });
}

function audit() {
    sendCommand({ type: 'audit' });
}

function nextAction() {
    sendCommand({ type: 'next-action' });
}

function buyUncleBert() {
    sendCommand({ type: 'buy-uncle-bert' });
}

function actionsFinished() {
    sendCommand({ type: 'actions-finished' });
}

function initialize(st, sc) {
    store = st;
    sendCommand = sc;

    const unsubscribe = store.subscribe(
        () => {
            const state = store.getState();
            if (state.farm.player.name === state.farm.game.currentPlayer
                && !state.farm.ui.actionChangeHandled) {
                store.dispatch(markActionChangeHandled());
                nextAction();
            }
        });

    // mpDims.mouseX = e.clientX
    // window.onmousemove = e => store.
    //     dispatch(mpMouse(e.clientX, store.getState().farm.mpDims.mouseY));
    // document.addEventListener('keydown', keydown);
}

function keydown(e) {
    switch (e.key) {
    case 'r':
        roll();
        break;
    case 'e':
        endTurn();
        break;
    }
}
