// Copyright 2020 Thomas Hintz
//
// This file is part of the Alpha Centauri Farming project.
//
// The Alpha Centauri Farming project is free software: you can
// redistribute it and/or modify it under the terms of the GNU General
// Public License as published by the Free Software Foundation, either
// version 3 of the License, or (at your option) any later version.
//
// The Alpha Centauri Farming project is distributed in the hope that
// it will be useful, but WITHOUT ANY WARRANTY; without even the
// implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
// PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with the Alpha Centauri Farming project.  If not, see
// <https://www.gnu.org/licenses/>.

import { UPDATE_GAME, UPDATE_PLAYER, GAME_STATE, SET_SELECTED_CARD, SET_CARDS,
         SPACE_PUSH_PLAYER, SPACE_CLEAR_PLAYERS,
         SET_OLD_MESSAGES, MESSAGE_PANEL_SPACE, MP_MOUSE,
         SET_MP_DIMS, MOVE_PLAYER, SET_NEXT_ACTION, NEXT_UI_ACTION,
         MARK_ACTION_CHANGE_HANDLED, NEXT_UI_ACTION_SILENT, ALERT, ALERT_HANDLED
       } from './actionTypes.js'
import { GAME_STATES } from '../../constants.js'
import { spaceContent, corners } from 'game.js'

const spaces =
    [[corners[0], 'buy'],
     ['January', 'buy'],
     ['January', 'buy'],
     ['January', 'buy'],
     ['January', 'buy'],
     ['February','buy'],
     ['February', 'buy'],
     ['February', 'buy'],
     ['February', 'buy'],
     ['March', 'buy'],
     ['March', 'buy'],
     ['March', 'buy'],
     ['March', 'buy'],
     ['April', 'buy'],
     [corners[1], 'buy'],
     ['April', 'none'],
     ['April', 'none'],
     ['May', 'none'],
     ['May', 'none'],
     ['May', 'hay'],
     ['May', 'hay'],
     ['June', 'hay'],
     ['June', 'hay'],
     ['June', 'cherry'],
     ['June', 'cherry'],
     [corners[2], 'cherry'],
     ['July', 'hay'],
     ['July', 'hay'],
     ['July', 'hay'],
     ['July', 'wheat'],
     ['August', 'wheat'],
     ['August', 'wheat'],
     ['August', 'wheat'],
     ['August', 'wheat'],
     ['September', 'hay'],
     ['September', 'hay'],
     ['September', 'cows'],
     [corners[3], 'cows'],
     ['September', 'cows'],
     ['October', 'cows'],
     ['October', 'hay'],
     ['October', 'hay'],
     ['October', 'apple'],
     ['November', 'apple'],
     ['November', 'apple'],
     ['November', 'apple'],
     ['November', 'corn'],
     ['December', 'corn'],
     ['December', 'corn']]
      .map((s, i) => {
          return { month: s[0], description: spaceContent[i],
                   type: s[1], key: i, players: [] }});

const initialState = {
    player: { cash: 5000,
              lastCash: 5000,
              debt: 5000,
              spaces,
              state: GAME_STATES.turnEnded,
              assets: { hay: 10, grain: 10, fruit: 0, cows: 0, harvester: 0, tractor: 0 },
              color: '',
              name: '',
              ridges: { ridge1: 0, ridge2: 0, ridge3: 0, ridge4: 0 },
              space: 0,
              trade: {}
            },
    game: { auditThreshold: 250000,
            calledAudit: false,
            currentPlayer: '',
            messages: [],
            otherPlayers: [],
            state: GAME_STATES.preTurn,
            turn: 0,
            oldMessages: [] },
    ui: { card: { type: 'no-card', contents: '', total: 0 },
          cards: [],
          action: false,
          actionValue: null,
          nextAction: false,
          nextActionValue: null,
          actionChangeHandled: true,
          alert: false,
          alertHandled: false },
    spaces: spaces,
    space: null,
    // message panel dimenions
    mpDims: { mouseX: 0, mouseY: 0,
              minWidth: 0, minHeight: 0, maxWidth: 0, maxHeight: 0,
              padding: 8 },
    profile: false,
    profileTurns: 500
}

export default function(state = initialState, action) {
    switch (action.type) {
    case UPDATE_GAME:
        return { ...state, game: { ...state.game, ...action.update }};
    case UPDATE_PLAYER:
        return { ...state, player: action.update };
    case GAME_STATE:
        return { ...state, game: { ...state.game, state: action.state } };
    case SET_SELECTED_CARD:
        return { ...state, ui: { ...state.ui, card: action.card }};
    case SET_CARDS:
        return { ...state, ui: { ...state.ui, cards: action.cards }};
    case MOVE_PLAYER:
        return { ...state, spaces: state.spaces
                 .map((item, index) => {
                     if (index === action.newSpace &&
                         item.players.indexOf(action.player) === -1) {
                         return { ...item, players: [...item.players, action.player]};
                     } else if (index === action.oldSpace) {
                         return { ...item,
                                  players: item.players
                                    .filter(x => x !== action.player) };
                     }
                     return item;
                 })
               };
    case SET_OLD_MESSAGES:
        return { ...state, oldMessages: action.messages };
    case MESSAGE_PANEL_SPACE:
        return { ...state, space: action.space };
    case MP_MOUSE:
        return { ...state, mpDims: { ...state.mpDims,
                                     mouseX: action.mouseX, mouseY: action.mouseY }};
    case SET_MP_DIMS:
        return { ...state, mpDims: { ...state.mpDims,
                                     minWidth: action.minWidth,
                                     minHeight: action.minHeight,
                                     maxWidth: action.maxWidth,
                                     maxHeight: action.maxHeight }};
    case SET_NEXT_ACTION:
        return { ...state, ui: { ...state.ui, nextAction: action.action,
                                 nextActionValue: action.value }};
    case NEXT_UI_ACTION:
        return { ...state, ui: { ...state.ui, action: state.ui.nextAction,
                                 actionValue: state.ui.nextActionValue,
                                 actionChangeHandled: !state.ui.nextAction }};
    case NEXT_UI_ACTION_SILENT: // don't set actionChangeHandled
        return { ...state, ui: { ...state.ui, action: state.ui.nextAction,
                                 actionValue: state.ui.nextActionValue }};
    case MARK_ACTION_CHANGE_HANDLED:
        return { ...state, ui: { ...state.ui, actionChangeHandled: true }};
    case ALERT:
        return { ...state, ui: { ...state.ui,
                                 alert: action.value,
                                 alertHandled: action.value === false ? true : false }};
    case ALERT_HANDLED:
        return { ...state, ui: { ...state.ui, alertHandled: true }};
    default:
        return state;
    }
}
